@@ -1,6 +1,7 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
  * Copyright (C) 2007 Xavier Claessens <xclaesse@gmail.com>
+ * Copyright (C) 2007 Collabora Ltd.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -16,6 +17,8 @@
  * License along with this program; if not, write to the
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
+ * 
+ * Authors: Xavier Claessens <xclaesse@gmail.com>
  */
 
 #include <config.h>
@@ -189,7 +192,6 @@ static void                   contact_list_parse_presence_foreach   (guint
 static void                   contact_list_presences_table_foreach  (const gchar                   *state_str,
 								     GHashTable                    *presences_table,
 								     GossipPresence               **presence);
-static GossipPresenceState    contact_list_presence_state_from_str  (const gchar                   *str);
 
 enum {
 	CONTACT_ADDED,
@@ -309,7 +311,7 @@ empathy_contact_list_new (McAccount *account)
 
 	g_return_val_if_fail (MC_IS_ACCOUNT (account), NULL);
 
-	mc = empathy_session_get_mission_control ();
+	mc = mission_control_new (tp_get_bus ());
 
 	if (mission_control_get_connection_status (mc, account, NULL) != 0) {
 		/* The account is not connected, nothing to do. */
@@ -317,6 +319,7 @@ empathy_contact_list_new (McAccount *account)
 	}
 
 	tp_conn = mission_control_get_connection (mc, account, NULL);
+	g_object_unref (mc);
 	g_return_val_if_fail (tp_conn != NULL, NULL);
 
 	list = g_object_new (EMPATHY_TYPE_CONTACT_LIST, NULL);
@@ -1724,17 +1727,14 @@ contact_list_parse_presence_foreach (guint               handle,
 			      (GHFunc) contact_list_presences_table_foreach,
 			      &presence);
 
-	gossip_debug (DEBUG_DOMAIN, "Presence changed for %s (%d)",
+	gossip_debug (DEBUG_DOMAIN, "Presence changed for %s (%d) to %s (%d)",
 		      gossip_contact_get_name (contact),
-		      handle);
+		      handle,
+		      presence ? gossip_presence_get_status (presence) : "unset",
+		      presence ? gossip_presence_get_state (presence) : MC_PRESENCE_UNSET);
 
 	contact_list_block_contact (list, contact);
-	if (presence) {
-		gossip_contact_add_presence (contact, presence);
-		g_object_unref (presence);
-	} else {
-		g_object_set (contact, "presences", NULL, NULL);
-	}
+	gossip_contact_set_presence (contact, presence);
 	contact_list_unblock_contact (list, contact);
 }
 
@@ -1743,19 +1743,19 @@ contact_list_presences_table_foreach (const gchar     *state_str,
 				      GHashTable      *presences_table,
 				      GossipPresence **presence)
 {
-	GossipPresenceState  state;
-	const GValue        *message;
+	McPresence    state;
+	const GValue *message;
+
+	state = gossip_presence_state_from_str (state_str);
+	if ((state == MC_PRESENCE_UNSET) || (state == MC_PRESENCE_OFFLINE)) {
+		return;
+	}
 
 	if (*presence) {
 		g_object_unref (*presence);
 		*presence = NULL;
 	}
 
-	state = contact_list_presence_state_from_str (state_str);
-	if (state == GOSSIP_PRESENCE_STATE_UNAVAILABLE) {
-		return;
-	}
-
 	*presence = gossip_presence_new ();
 	gossip_presence_set_state (*presence, state);
 
@@ -1764,30 +1764,5 @@ contact_list_presences_table_foreach (const gchar     *state_str,
 		gossip_presence_set_status (*presence,
 					    g_value_get_string (message));
 	}
-
-	gossip_presence_set_resource (*presence, "");
-}
-
-static GossipPresenceState
-contact_list_presence_state_from_str (const gchar *str)
-{
-	if (strcmp (str, "available") == 0) {
-		return GOSSIP_PRESENCE_STATE_AVAILABLE;
-	} else if ((strcmp (str, "dnd") == 0) || (strcmp (str, "busy") == 0)) {
-		return GOSSIP_PRESENCE_STATE_BUSY;
-	} else if ((strcmp (str, "away") == 0) || (strcmp (str, "brb") == 0)) {
-		return GOSSIP_PRESENCE_STATE_AWAY;
-	} else if (strcmp (str, "xa") == 0) {
-		return GOSSIP_PRESENCE_STATE_EXT_AWAY;
-	} else if (strcmp (str, "hidden") == 0) {
-		return GOSSIP_PRESENCE_STATE_HIDDEN;
-	} else if (strcmp (str, "offline") == 0) {
-		return GOSSIP_PRESENCE_STATE_UNAVAILABLE;
-	} else if (strcmp (str, "chat") == 0) {
-		/* We don't support chat, so treat it like available. */
-		return GOSSIP_PRESENCE_STATE_AVAILABLE;
-	}
-
-	return GOSSIP_PRESENCE_STATE_AVAILABLE;
 }
 
